﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using System.IO;
using ThinkGeo.MapSuite.DesktopEdition;

namespace SphericalMercatorGridTest
{
    public partial class Form1 : Form
    {

        private Proj4Projection proj4Projection = new Proj4Projection();

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // setup the geodetic
            winformsMap1.MapUnit = GeographyUnit.Meter;

            
            proj4Projection.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
            proj4Projection.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();

            GraticuleAdornmentLayer graticuleAdornmentLayer = new GraticuleAdornmentLayer();
            graticuleAdornmentLayer.GraticuleLineStyle.OuterPen.Color = GeoColor.FromArgb(125, GeoColor.StandardColors.White);
            graticuleAdornmentLayer.GraticuleTextFont = new GeoFont("Arial", 8, DrawingFontStyles.Bold);
            graticuleAdornmentLayer.Projection = proj4Projection;


            winformsMap1.AdornmentOverlay.Layers.Add("graticule", graticuleAdornmentLayer);

            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.StandardColors.LightGray);

            // Setup an overlay to display extents and points
            InMemoryFeatureLayer layoutInMemoryFeatureLayer = new InMemoryFeatureLayer();
            GeoColor geoBaseColor = GeoColor.FromArgb(75, GeoColor.SimpleColors.Yellow);
            layoutInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle =
                AreaStyles.CreateSimpleAreaStyle(geoBaseColor, geoBaseColor, 2);
            layoutInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle =
                LineStyles.CreateSimpleLineStyle(geoBaseColor, 2, false);
            layoutInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay layoutOverLayer = new LayerOverlay();
            layoutOverLayer.Layers.Add("LayoutInMemoryFeatureLayer", layoutInMemoryFeatureLayer);
            layoutOverLayer.Layers["LayoutInMemoryFeatureLayer"].Name = "LayoutInMemoryFeatureLayer";
            winformsMap1.Overlays.Add("LayoutOverlay", layoutOverLayer);
            winformsMap1.Overlays["LayoutOverlay"].Name = "LayoutOverlay";

            // Setup the Grid Overlay
            InMemoryFeatureLayer gridLayer = new InMemoryFeatureLayer();
            gridLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Cross, 
                GeoColor.StandardColors.Black, GeoColor.StandardColors.LightGray, 1, 4);
            gridLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay gridOverlay = new LayerOverlay();
            gridOverlay.Layers.Add("GridData", gridLayer);
            gridOverlay.Layers["GridData"].Name = "GridData";
            winformsMap1.Overlays.Add("GridOverlay", gridOverlay);
            winformsMap1.Overlays["GridOverlay"].Name = "GridOverlay";

            // Setup the isolines Overlay
            InMemoryFeatureLayer isoLayer = new InMemoryFeatureLayer();
            isoLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Cross, 
                GeoColor.StandardColors.Black, GeoColor.StandardColors.LightGray, 1, 4);
            isoLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay isoOverlay = new LayerOverlay();
            isoOverlay.Layers.Add("IsolineData", isoLayer);
            isoOverlay.Layers["IsolineData"].Name = "IsolineData";
            winformsMap1.Overlays.Add("IsolineOverlay", isoOverlay);
            winformsMap1.Overlays["IsolineOverlay"].Name = "IsolineOverlay";

            winformsMap1.Refresh();

        }

        private static void DisplayExtents(WinformsMap winformsMap, Dictionary<PointShape, double> bathyPoints, double areaBuffer, Proj4Projection proj4Projection)
        {

            double minX = bathyPoints.Keys.Min(p => p.X) - areaBuffer;
            double minY = bathyPoints.Keys.Min(p => p.Y) - areaBuffer;
            double maxX = bathyPoints.Keys.Max(p => p.X) + areaBuffer;
            double maxY = bathyPoints.Keys.Max(p => p.Y) + areaBuffer;

            LayerOverlay layoutOverlay = (LayerOverlay)winformsMap.Overlays["LayoutOverlay"];

            InMemoryFeatureLayer imfl = new InMemoryFeatureLayer();

            imfl.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.Black, 3, false);
            imfl.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            List<Vertex> corners = new List<Vertex>();

            Vertex vertex1 = new Vertex(minX, minY);
            Vertex vertex2 = new Vertex(maxX, minY);
            Vertex vertex3 = new Vertex(maxX, maxY);
            Vertex vertex4 = new Vertex(minX, maxY);

            vertex1 = GridFunctions.ConvertUTMToSMCoordinate(vertex1, proj4Projection);
            vertex2 = GridFunctions.ConvertUTMToSMCoordinate(vertex2, proj4Projection);
            vertex3 = GridFunctions.ConvertUTMToSMCoordinate(vertex3, proj4Projection);
            vertex4 = GridFunctions.ConvertUTMToSMCoordinate(vertex4, proj4Projection);

            corners.Add(vertex1);
            corners.Add(vertex2);
            corners.Add(vertex3);
            corners.Add(vertex4);
            corners.Add(vertex1);

            LineShape border = new LineShape(corners);

            imfl.InternalFeatures.Add(new Feature(border));

            layoutOverlay.Layers.Add(imfl);
        }

        private static void DisplayPoints(WinformsMap winformsMap, Dictionary<PointShape, double> bathyPoints)
        {

            // Get the points into a shape
            MultipointShape allPoints = new MultipointShape();

            foreach (KeyValuePair<PointShape, double> kvp in bathyPoints)
            {
                allPoints.Points.Add(kvp.Key);
            }

            LayerOverlay layoutOverlay = (LayerOverlay)winformsMap.Overlays["LayoutOverlay"];

            InMemoryFeatureLayer imfl = new InMemoryFeatureLayer();

            imfl.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimpleSquareStyle(GeoColor.StandardColors.Black, 5);
            imfl.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            imfl.InternalFeatures.Add(new Feature(allPoints));

            layoutOverlay.Layers.Add(imfl);

        }

        private static void LoadGrid(WinformsMap winformsMap, Proj4Projection proj4Projection, string filename, string gridFilename)
        {

            // Return points in UTM
            Dictionary<PointShape, double> bathyPoints = GridFunctions.GetBathyPoints(filename, proj4Projection);

            // Make an SM bathy points dictionary as well
            Dictionary<PointShape, double> smBathyPoints = new Dictionary<PointShape, double>();
            foreach (KeyValuePair<PointShape, double> kvp in bathyPoints)
            {
                PointShape convertedPoint = GridFunctions.ConvertUTMToSMCoordinate(kvp.Key, proj4Projection);
                smBathyPoints.Add(convertedPoint, kvp.Value);
            }

            // Display the extents - send UTM because we need to add the areaBuffer to the grid
            DisplayExtents(winformsMap, bathyPoints, 21000, proj4Projection);

            // Send the points to display in SM
            DisplayPoints(winformsMap, smBathyPoints);

            // Establish the grid overlay
            LayerOverlay gridOverlay = (LayerOverlay)winformsMap.Overlays["GridOverlay"];
            gridOverlay.Name = "GridOverlay";

            // #########################################################
            // HERE IS THE TEST. SEND ONE OPTION TO THE SCREEN
            // Send UTM points but be aware that the grid
            //GridFeatureLayer tempLayer = GridFunctions.MakeGenericBathyGrid(winformsMap, gridFilename,
            //    proj4Projection, 21000, bathyPoints);

            // or send SM points
            GridFeatureLayer tempLayer = GridFunctions.MakeGenericBathyGrid(winformsMap, gridFilename,
                proj4Projection, 21000, smBathyPoints);
            // #########################################################


            tempLayer.Name = new FileInfo(gridFilename).Name;
            gridOverlay.Layers.Add(tempLayer.Name, tempLayer.CloneDeep());


        }

        private void btnLoadFile_Click(object sender, EventArgs e)
        {

            // Specify the input utm coordinate transform
            proj4Projection.InternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();
            proj4Projection.ExternalProjectionParametersString = string.Format("+proj=utm +zone={0} +{1} +ellps=WGS84 +datum=WGS84 +units=m +no_defs", "35", "north");//UTM

            // Input data are in UTM 35N

            // Load the text file as a grid
            string textFilename = @"..\..\data\TestData_North.txt";
 
            // Now return a grid feature layer from the points and add it to the overlay for display.
            string gridFilename = @"..\..\data\testNorth.grd";

            LoadGrid(winformsMap1, proj4Projection, textFilename, gridFilename);

            winformsMap1.Refresh();

        }

        private void btnLoadCentralFile_Click(object sender, EventArgs e)
        {

            // Specify the input utm coordinate transform
            proj4Projection.InternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();
            proj4Projection.ExternalProjectionParametersString = string.Format("+proj=utm +zone={0} +{1} +ellps=WGS84 +datum=WGS84 +units=m +no_defs", "35", "north");//UTM

            // Input data are in UTM 35N

            // Load the text file as a grid
            string textFilename = @"..\..\data\TestData_Central.txt";

            // Now return a grid feature layer from the points and add it to the overlay for display.
            string gridFilename = @"..\..\data\testCentral.grd";

            LoadGrid(winformsMap1, proj4Projection, textFilename, gridFilename);

            winformsMap1.Refresh();
        }
    }
}
